Package: limma
Version: 3.68.5
Date: 2026-08-10
Title: Linear Models for Microarray and Omics Data
Description: Data analysis, linear models and differential expression for omics data.
Authors@R: c(person(given = "Gordon",
                    family = "Smyth",
                    role = c("cre", "aut"),
                    email = "smyth@wehi.edu.au",
                    comment = c(ORCID = "0000-0001-9221-2892")),
             person(given = "Yifang",
                    family = "Hu",
                    role = "ctb"),
             person(given = "Matthew",
                    family = "Ritchie",
                    role = "ctb"),
             person(given = "Jeremy",
                    family = "Silver",
                    role = "ctb"),
             person(given = "James",
                    family = "Wettenhall",
                    role = "ctb"),
             person(given = "Davis",
                    family = "McCarthy",
                    role = "ctb"),
             person(given = "Di",
                    family = "Wu",
                    role = "ctb"),
             person(given = "Wei",
                    family = "Shi",
                    role = "ctb"),
             person(given = "Belinda",
                    family = "Phipson",
                    role = "ctb"),
             person(given = "Aaron",
                    family = "Lun",
                    role = "ctb"),
             person(given = "Natalie",
                    family = "Thorne",
                    role = "ctb"),
             person(given = "Alicia",
                    family = "Oshlack",
                    role = "ctb"),
             person(given = "Carolyn",
                    family = "de Graaf",
                    role = "ctb"),
             person(given = "Yunshun",
                    family = "Chen",
                    role = "ctb"),
             person(given = "Goknur",
                    family = "Giner",
                    role = "ctb"),
             person(given = "Mette",
                    family = "Langaas",
                    role = "ctb"),
             person(given = "Egil",
                    family = "Ferkingstad",
                    role = "ctb"),
             person(given = "Marcus",
                    family = "Davy",
                    role = "ctb"),
             person(given = "Francois",
                    family = "Pepin",
                    role = "ctb"),
             person(given = "Dongseok",
                    family = "Choi",
                    role = "ctb"),
             person(given = "Charity",
                    family = "Law",
                    role = "ctb"),
             person(given = "Mengbo",
                    family = "Li",
                    role = "ctb"),
             person(given = "Lizhong",
                    family = "Chen",
                    role = "ctb",
                    comment = c(ORCID = "0000-0002-8326-6781"))
             )
License: GPL (>=2)
Depends: R (>= 3.6.0)
Imports: grDevices, graphics, stats, utils, methods, statmod
Suggests: BiasedUrn, ellipse, gplots, knitr, locfit, MASS, splines,
        affy, AnnotationDbi, Biobase, BiocStyle, GO.db, illuminaio,
        org.Hs.eg.db, vsn
VignetteBuilder: knitr
URL: https://bioinf.wehi.edu.au/limma/
biocViews: ExonArray, GeneExpression, Transcription,
        AlternativeSplicing, DifferentialExpression,
        DifferentialSplicing, GeneSetEnrichment, DataImport, Bayesian,
        Clustering, Regression, TimeCourse, Microarray, MicroRNAArray,
        mRNAMicroarray, OneChannel, ProprietaryPlatforms, TwoChannel,
        Sequencing, RNASeq, BatchEffect, MultipleComparison,
        Normalization, Preprocessing, QualityControl,
        BiomedicalInformatics, CellBiology, Cheminformatics,
        Epigenetics, FunctionalGenomics, Genetics, ImmunoOncology,
        Metabolomics, Proteomics, SystemsBiology, Transcriptomics
git_url: https://git.bioconductor.org/packages/limma
git_branch: RELEASE_3_23
git_last_commit: 825d1c8
git_last_commit_date: 2026-08-09
Repository: Bioconductor 3.23
Date/Publication: 2026-08-10
NeedsCompilation: yes
Packaged: 2026-08-10 22:32:02 UTC; biocbuild
Author: Gordon Smyth [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-9221-2892>),
  Yifang Hu [ctb],
  Matthew Ritchie [ctb],
  Jeremy Silver [ctb],
  James Wettenhall [ctb],
  Davis McCarthy [ctb],
  Di Wu [ctb],
  Wei Shi [ctb],
  Belinda Phipson [ctb],
  Aaron Lun [ctb],
  Natalie Thorne [ctb],
  Alicia Oshlack [ctb],
  Carolyn de Graaf [ctb],
  Yunshun Chen [ctb],
  Goknur Giner [ctb],
  Mette Langaas [ctb],
  Egil Ferkingstad [ctb],
  Marcus Davy [ctb],
  Francois Pepin [ctb],
  Dongseok Choi [ctb],
  Charity Law [ctb],
  Mengbo Li [ctb],
  Lizhong Chen [ctb] (ORCID: <https://orcid.org/0000-0002-8326-6781>)
Maintainer: Gordon Smyth <smyth@wehi.edu.au>
