---
title: A brief introduction to limma
date: "23 October 2004 (last revised 11 June 2023)"
output:
  BiocStyle::html_document:
    toc: FALSE
    number_sections: FALSE
vignette: >
  %\VignetteIndexEntry{A brief introduction to limma}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  prompt = TRUE,
  comment = NA
)
```

# What is it?

Limma is an R package for the analysis of gene expression data, especially the use of linear models for analysing designed experiments and the assessment of differential expression.
Limma provides the ability to analyse comparisons between many RNA targets simultaneously in arbitrary complicated designed experiments.
Empirical Bayesian methods are used to provide stable results even when the number of arrays is small.
The normalization and background correction functions are provided for microarrays and similar technologies.
The linear model and differential expression functions apply to a wide variety of gene expression technologies including microarrays (single-channel or two-color), quantitative PCR, RNA-seq or proteomics.

# How to get help

The edgeR User's Guide is available by
```{r, eval=FALSE, echo=TRUE}
library(limma)
limmaRUsersGuide()
```
or alternatively from the [limma landing page](https://bioconductor.org/packages/limma).

Documentation for specific functions is available through the usual R help system, e.g., `?lmFit`.
Further questions about the package should be directed to the [Bioconductor support site](https://support.bioconductor.org).

# Further reading

Ritchie ME, Phipson B, Wu D, Hu Y, Law CW, Shi W, Smyth GK (2015). limma powers differential expression analyses for RNA-sequencing and microarray studies. *Nucleic Acids Research* 43, e47. [doi:10.1093/nar/gkv007](https://doi.org/10.1093/nar/gkv007)

Phipson B, Lee S, Majewski IJ, Alexander WS, and Smyth GK (2016). Robust hyperparameter estimation protects against hypervariable genes and improves power to detect differential expression. *Annals of Applied Statistics* 10, 946-963. [doi:10.1214/16-AOAS920](10.1214/16-AOAS920)

Law CW, Chen Y, Shi W, Smyth GK (2014). Voom: precision weights unlock linear model analysis tools for RNA-seq read counts. Genome Biology 15, R29. [doi:10.1186/gb-2014-15-2-r29](https://doi.org/10.1186/gb-2014-15-2-r29). See also the Preprint Version at https://gksmyth.github.io/pubs/VoomPreprint.pdf incorporating some notational corrections.

Law CW, Alhamdoosh M, Su S, Dong X, Tian L, Smyth GK, Ritchie ME (2016). RNA-seq analysis is easy as 1-2-3 with limma, Glimma and edgeR. *F1000Research* 5, 1408.
[doi:10.12688/f1000research.9005.3](https://doi.org/10.12688/f1000research.9005.3)

Law CW, Zeglinski K, Dong X, Alhamdoosh M, Smyth GK, Ritchie ME (2020). A guide to creating design matrices for gene expression experiments. *F1000Research* 9, 1444.
[doi:10.12688/f1000research.27893.1](https://doi.org/10.12688/f1000research.27893.1)

